package future_good

import (
	"encoding/json"
	"eta/eta_chart_lib/models"
	"eta/eta_chart_lib/utils"
	"fmt"
	"io/ioutil"
	"net/http"
	"strings"
)

// AddEdbData 添加指标数据
func AddEdbData(futureGoodEdbCode string) (resp *models.BaseResponse, err error) {
	param := make(map[string]interface{})
	param["EdbCode"] = futureGoodEdbCode
	urlStr := `future_good/add`
	resp, err = postRefreshEdbData(param, urlStr)
	return
}

// RefreshEdbData 刷新指标数据
func RefreshEdbData(futureGoodEdbInfoId int, futureGoodEdbCode, startDate string) (resp *models.BaseResponse, err error) {
	param := make(map[string]interface{})
	param["FutureGoodEdbCode"] = futureGoodEdbCode
	param["FutureGoodEdbInfoId"] = futureGoodEdbInfoId
	param["StartDate"] = startDate
	urlStr := `future_good/refresh`
	resp, err = postRefreshEdbData(param, urlStr)
	return
}

// RefreshEdbRelation 刷新商品期货指标相关的数据
func RefreshEdbRelation(chartInfoId int) (resp *models.BaseResponse, err error) {
	param := make(map[string]interface{})
	param["ChartInfoId"] = chartInfoId
	urlStr := `future_good/relation/refresh`
	resp, err = postRefreshEdbData(param, urlStr)
	return
}

// postRefreshEdbData 刷新指标数据
func postRefreshEdbData(param map[string]interface{}, urlStr string) (resp *models.BaseResponse, err error) {
	postUrl := utils.EDB_LIB_URL + urlStr
	postData, err := json.Marshal(param)
	if err != nil {
		return
	}
	result, err := HttpPost(postUrl, string(postData), "application/json")
	if err != nil {
		return
	}
	utils.FileLog.Info("postRefreshEdbData:" + postUrl + ";" + string(postData) + ";result:" + string(result))
	err = json.Unmarshal(result, &resp)
	if err != nil {
		return
	}
	return resp, nil
}

func HttpPost(url, postData string, params ...string) ([]byte, error) {
	body := ioutil.NopCloser(strings.NewReader(postData))
	client := &http.Client{}
	req, err := http.NewRequest("POST", url, body)
	if err != nil {
		return nil, err
	}
	contentType := "application/x-www-form-urlencoded;charset=utf-8"
	if len(params) > 0 && params[0] != "" {
		contentType = params[0]
	}
	req.Header.Set("Content-Type", contentType)
	req.Header.Set("authorization", utils.MD5(utils.APP_EDB_LIB_NAME_EN+utils.EDB_LIB_Md5_KEY))
	resp, err := client.Do(req)
	defer resp.Body.Close()
	b, err := ioutil.ReadAll(resp.Body)
	fmt.Println("HttpPost:" + string(b))
	return b, err
}
